import { Body, Controller, Inject, Post } from '@midwayjs/core';
import { Context } from '@midwayjs/koa';
import { BizCode } from '../biz/code';
import { webSign } from '../config/base.config';
import { CourseCreateDTO } from '../dto/course.dto';
import { ChapterService } from '../service/chapter.service';
import { CourseService } from '../service/course.service';
import { GuideService } from '../service/guide.service';
import { UserService } from '../service/user.service';
import { decodeToken } from '../util/encrypt';

@Controller('/course')
export class CourseController {
  @Inject()
  ctx: Context;

  @Inject()
  courseService: CourseService;

  @Inject()
  chapterService: ChapterService;

  @Inject()
  guideService: GuideService;

  @Inject()
  userService: UserService;

  @Post('/create')
  async create(@Body() param: CourseCreateDTO) {
    try {
      const { course_chapterList, course_guide, ...rest } = param;
      const courseId = await this.courseService.create({ ...rest });
      await this.chapterService.create(
        course_chapterList.map((i: any) => ({
          ...i,
          chapter_course_id: courseId,
        }))
      );
      await this.guideService.create({
        ...course_guide,
        guide_course_id: courseId,
      });
      return { code: BizCode.OK };
    } catch (error) {
      this.ctx.logger.error(error);
      return {
        code: BizCode.ERROR,
        msg: `[error] course/create error happened`,
      };
    }
  }

  @Post('/select/all')
  async selectAll() {
    const courseList = await this.courseService.selectAll();
    return { code: BizCode.OK, data: courseList };
  }

  @Post('/detail/select')
  async selectDetailByCourseId(@Body() params) {
    const { course_id } = params;
    try {
      const token = this.ctx.cookies.get(webSign);
      const { user_login } = decodeToken(token);
      const user = await this.userService.select({ user_login });
      // 用户订阅鉴权
      if (!user.user_sub)
        return { code: BizCode.AUTH, msg: '无权访问订阅课程' };
      const course = await this.courseService.select({ course_id });
      const chapterList = await this.chapterService.select(course_id);
      const guide = await this.guideService.select(course_id);
      return { code: BizCode.OK, data: { chapterList, guide, course } };
    } catch (error) {
      return { code: BizCode.ERROR, msg: '[error] /chapter/select error' };
    }
  }
}
